//======================================================================
// File:		dso.h
// Author:	Matthias Toussaint
// Created:	Mon Jun 10 17:13:44 CEST 2002
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2000-2002 Matthias Toussaint
//======================================================================

#ifndef DSO_HH
#define DSO_HH

#include <qthread.h>

/** Abstract base class for DSO protocoll
*/
class Dso : public QThread
{
public:
  enum TimeBase
  {
    TB100ms = 0,
    TB50ms,
    TB20ms,
    TB10ms,
    TB5ms,
    TB2ms,
    TB1ms,
    TB500us,
    TB200us,
    TB100us,
    TB50us,
    TB20us,
    TB10us,
    TB5us,
    TB2us,
    TB1us,
    TB05us,
    TB02us,
    TB01us
  };
    
  enum VoltsDiv
  {
    VD5v = 0,
    VD2v,
    VD1v,
    VD500mv,
    VD200mv,
    VD100mv,
    VD50mv,
    VD20mv,        
    VD10mv
  };
   
  enum Model
  {
    Simulator = 0,
    PCS64i,
    PCS32,
    PCS100,
    PCS500,
    Unknown
  };
    
  Dso( Model );
  virtual ~Dso();
   
  Dso::Model const model() const { return m_model; }
  
  virtual void run() = 0;
  
  /// Timebase in us
  virtual void setTimeBase( Dso::TimeBase );
  /// volts/div in mv
  virtual void setVoltsDiv( int, Dso::VoltsDiv );
  /// enable trigger
  virtual void setTriggerEnabled( bool );
  /// should be 1 or 2
  virtual void setTriggerChannel( int );
  /// trigger on raising edge, else on falling
  virtual void setTriggerRaising( bool );
  /// trigger level
  virtual void setTriggerLevel( int );
  /// connect to the selected hardware (port or whatsoever)
  virtual bool connectToHardware() = 0;
  
  inline void setFastAcq( bool on ) { m_fastAcq = on; }
  inline bool fastAcq() const { return m_fastAcq; }
  inline void setAcqLength( int len ) { m_acqLength = len; }
  inline int acqLength() const { return m_acqLength; }
  inline void setAcqOffset( int off ) { m_acqOffset = off; }
  inline int acqOffset() const { return m_acqOffset; }
  inline const int preTriggerSize() const { return m_preTriggerSize; }
  
  virtual void setChannelEnable( int, bool ) {};
  
  void lock();
  void unlock();
  
  virtual const bool hasDcOffset() const = 0;
  virtual const int  numChannel() const = 0;
  virtual const int  triggerBits() const = 0;
  virtual const int  bits() const = 0;
  
  inline const int data( const int, const int ) const;
  inline const float samplingFrequency() const { return m_samplingFrequency; }
  inline const float timeBase() const { return m_timeBaseSec; }  
  inline const bool triggerOk() const { return m_triggerOk; }  
  inline const int triggerChannel() const { return m_triggerChannel; }
  inline const int triggerLevel() const { return m_triggerLevel; }
  inline const int step() const { return m_step; }
  inline const int numSamples() const { return m_numSamples; }
  inline const float voltsDiv( int channel ) const { return m_voltsDivVolts[channel]; }
  inline bool hasNewData() const { return m_hasNewData; }
  inline void resetNewData() { m_hasNewData = false; }
  
protected:
  Model     m_model;
  unsigned *m_outBuffer[2];
  unsigned *m_buffer[2];
  TimeBase  m_timeBase;
  float     m_timeBaseSec;
  VoltsDiv  m_voltsDiv[2];
  float     m_voltsDivVolts[2];
  bool      m_triggerEnabled;
  int       m_triggerChannel;
  bool      m_triggerRaising;
  int       m_triggerLevel;
  QMutex    m_mutex;
  bool      m_triggerOk;
  int       m_step;
  float     m_samplingFrequency;
  int       m_numSamples;
  int       m_preTriggerSize;
  bool      m_hasNewData;
  bool      m_fastAcq;
  int       m_acqLength;
  int       m_acqOffset;
  
};

const int
Dso::data( const int channel, const int index ) const
{
  return m_outBuffer[channel][index];
}

#endif // DSO_HH
