//======================================================================
// File:		simulator.cpp
// Author:	Matthias Toussaint
// Created:	Sat Jul  6 21:07:22 CEST 2002
//----------------------------------------------------------------------
// Permission to use, copy, modify, and distribute this software and its
// documentation  for any  purpose and  without fee is  hereby  granted,
// provided  that below copyright notice appear  in all copies  and that
// both  that  copyright  notice and  this permission  notice  appear in
// supporting documentation.
// 
// This  file is  provided AS IS  with no  warranties  of any kind.  The
// author shall  have no liability  with respect  to the infringement of
// copyrights, trade  secrets  or any patents by  this file  or any part
// thereof.  In no event will the author be liable  for any lost revenue
// or profits or other special, indirect and consequential damages.
//----------------------------------------------------------------------
// (c) 2000-2002 Matthias Toussaint
//======================================================================

#include <simulator.h>
#include <math.h>

#include <iostream>

Simulator::Simulator() :
  Dso( Dso::Simulator )
{
  m_outBuffer[0] = new unsigned [4096];
  m_outBuffer[1] = new unsigned [4096];
  
  m_buffer[0] = new unsigned [4096];
  m_buffer[1] = new unsigned [4096];
  
  // use ring buffer to simulate pre-trigger
  // 1024 values pre trigger memory 
  // seems to be what Velleman PCi64 does
  //
  m_ringBuffer[0] = new unsigned [1024];
  m_ringBuffer[1] = new unsigned [1024];
  
  m_freq[0] = 100.;
  m_freq[1] = 100.;
  
  m_waveform[0] = Sine;
  m_waveform[1] = Sine;
  
  m_threshold[0] = 0.0;
  
  m_addNoise[0] = false;
  m_addNoise[1] = false;
  
  m_vPp[0] = 0.1;
  m_vPp[1] = 0.1;
  
  m_cnt = 0;
  
  m_sawtoothValue[0] = m_sawtoothValue[1] = 0.0;
  
  m_acqLength = 4096;
  m_numSamples = 4096;
    
  setTimeBase( Dso::TB1ms );
  setVoltsDiv( 0, Dso::VD5v );
  setVoltsDiv( 1, Dso::VD5v );
  setTriggerEnabled( false );
  setTriggerChannel( 0 );
  setTriggerRaising( true );
  setTriggerLevel( 128 );

  
  for (int i=0; i<4096; ++i)
  {
    m_outBuffer[0][i] = m_outBuffer[1][i] = 127;
  }
}

Simulator::~Simulator()
{
}

void
Simulator::waveformChangedSLOT( unsigned channel, Simulator::Waveform wf )
{
  m_waveform[channel] = wf;
}

void
Simulator::frequencyChangedSLOT( unsigned channel, float freq )
{
  m_freq[channel] = freq;
}

void
Simulator::vppChangedSLOT( unsigned channel, float vpp )
{
  m_vPp[channel] = vpp;
}

void
Simulator::noiseToggledSLOT( unsigned channel, bool on )
{
  m_addNoise[channel] = on;
}

void
Simulator::run()
{
  //while (1)
  {
    //if (!m_hasNewData)
    {
      setupSimulator();

      // first fill buffer
      //
      for (int i=0; i<1024; ++i)
      {
        m_ringBuffer[0][i] = sample( 0 );
        m_ringBuffer[1][i] = sample( 1 );

        ++m_cnt;
      }

      // now wait for trigger condition
      // 
      if (m_triggerEnabled)
      {
        if (m_triggerRaising)
        {
          for (int i=0; i<4096; ++i)
          {
            m_ringBuffer[0][m_ringBufferPointer] = sample( 0 );
            m_ringBuffer[1][m_ringBufferPointer] = sample( 1 );

            ++m_cnt;
            ++m_ringBufferPointer;
            m_ringBufferPointer %= 1024;

            if (raisingTrigger())
            {
              break;
            }
          }
        }
        else
        {
          for (int i=0; i<4096; ++i)
          {
            m_ringBuffer[0][m_ringBufferPointer] = sample( 0 );
            m_ringBuffer[1][m_ringBufferPointer] = sample( 1 );

            ++m_cnt;
            ++m_ringBufferPointer;
            m_ringBufferPointer %= 1024;

            if (fallingTrigger())
            {
              break;
            }
          }
        }
      }

      // copy ring buffer into first values
      //
      for (int i=0; i<1024; ++i)
      {
        m_buffer[0][i] = m_ringBuffer[0][m_ringBufferPointer];
        m_buffer[1][i] = m_ringBuffer[1][m_ringBufferPointer];

        ++m_ringBufferPointer;
        m_ringBufferPointer %= 1024;
      }

      // fill remaining samples
      //
      for (int i=1024; i<4096; ++i)
      {
        m_buffer[0][i] = sample( 0 ); 
        m_buffer[1][i] = sample( 1 ); 

        ++m_cnt;
      }

      // synchronize with reading from widget
      //
      m_mutex.lock();

      memcpy( m_outBuffer[0], m_buffer[0], 4096*sizeof(unsigned) );
      memcpy( m_outBuffer[1], m_buffer[1], 4096*sizeof(unsigned) );

      m_hasNewData = true;

      m_mutex.unlock();
    }
    
    //msleep( 20 );
  }
}

void
Simulator::setupSimulator()
{
    //float samplesPerPeriod[2];
    float ts = 1./160e3;
    float tch1 = 1./(float)m_freq[0];
    float tch2 = 1./(float)m_freq[1];

    //m_cnt = (int)((float)random() / (float)RAND_MAX * 2048);
    
    m_ringBufferPointer = 0;
    
    switch (m_timeBase)
    {
    case TB01us:
      ts = 1./32e6;
      break;  
    case TB02us:
      ts = 1./32e6;
      break;  
    case TB05us:
      ts = 1./32e6;
      break;  
    case TB1us:
      ts = 1./32e6;
      break;  
    case TB2us:
      ts = 1./32e6;
      break;  
    case TB5us:
      ts = 1./32e6;
      break;  
    case TB10us:
      ts = 1./16e6;
      break;
    case TB20us:
      ts = 1./8e6;
      break;  
    case TB50us:
      ts = 1./3.2e6;
      break;  
    case TB100us:
      ts = 1./1.6e6;
      break;  
    case TB200us:
      ts = 1./800e3;
      break;  
    case TB500us:
      ts = 1./320e3;
      break;  
    case TB1ms:
      ts = 1./160e3;
      break;  
    case TB2ms:
      ts = 1./80e3;
      break;  
    case TB5ms:
      ts = 1./32e3;
      break;  
    case TB10ms:
      ts = 1./16e3;
      break;  
    case TB20ms:
      ts = 1./8e3;
      break;  
    case TB50ms:
      ts = 1./3.2e3;
      break;  
    case TB100ms:
      ts = 1./1.6e3;
      break;  
    }

    float aFac[2];
    
    switch (m_voltsDiv[0])
    {
    case VD5v:
      aFac[0] = 32./5.;  
      break;
    case VD2v:
      aFac[0] = 32./2.;  
      break;
    case VD1v:
      aFac[0] = 32.;  
      break;
    case VD500mv:
      aFac[0] = 32./0.5;  
      break;
    case VD200mv:
      aFac[0] = 32./0.2;  
      break;
    case VD100mv:
      aFac[0] = 32./0.1;  
      break;
    case VD50mv:
      aFac[0] = 32./0.05;  
      break;
    case VD20mv:
      aFac[0] = 32./0.02;  
      break;
    case VD10mv:
      aFac[0] = 32./0.01;  
      break;
    }
    
    switch (m_voltsDiv[1])
    {
    case VD5v:
      aFac[1] = 32./5.;  
      break;
    case VD2v:
      aFac[1] = 32./2.;  
      break;
    case VD1v:
      aFac[1] = 32.;  
      break;
    case VD500mv:
      aFac[1] = 32./0.5;  
      break;
    case VD200mv:
      aFac[1] = 32./0.2;  
      break;
    case VD100mv:
      aFac[1] = 32./0.1;  
      break;
    case VD50mv:
      aFac[1] = 32./0.05;  
      break;
    case VD20mv:
      aFac[1] = 32./0.02;
      break;
    case VD10mv:
      aFac[1] = 32./0.01;  
      break;
    }
    
    m_samplesPerPeriod[0] = tch1 / ts;
    m_samplesPerPeriod[1] = tch2 / ts;

    m_div[0] = 2.*M_PI/m_samplesPerPeriod[0];
    m_div[1] = 2.*M_PI/m_samplesPerPeriod[1];

    m_a[0] = m_vPp[0] / 2. * aFac[0];
    m_a[1] = m_vPp[1] / 2. * aFac[1];

    m_sawtoothStep[0] = 2.*m_a[0] / m_samplesPerPeriod[0];
    m_sawtoothStep[1] = 2.*m_a[1] / m_samplesPerPeriod[1];
    
    m_sawtoothOff[0] = 128.-m_a[0];
    m_sawtoothOff[1] = 128.-m_a[1];
}

unsigned 
Simulator::sample( int channel )
{
  switch (m_waveform[channel])
  {
  case Sine:
    return clamp( 128. + m_a[channel] * sin( (float)m_cnt*m_div[channel] ) + noise(channel) );
    break;

  case Square:
    return clamp( 128. + (sin( (float)m_cnt*m_div[channel] ) >= 0. ? m_a[channel] : -m_a[channel]) 
        + noise(channel) );
    break;

  case Sawtooth:
    m_sawtoothValue[channel] += m_sawtoothStep[channel];
    m_sawtoothValue[channel] = fmod( m_sawtoothValue[channel], 2.*m_a[channel] );

    return clamp( m_sawtoothOff[channel] + m_sawtoothValue[channel] + noise(channel) );
    break;      
  }
  
  return 0;
}

bool
Simulator::raisingTrigger() const
{
  const unsigned level = m_triggerLevel & 0xf0;

  if (m_ringBuffer[m_triggerChannel][m_ringBufferPointer] >= level)
  {
    int pointer = m_ringBufferPointer;
    pointer += 1022;
    pointer %= 1024;
    
    if (m_ringBuffer[m_triggerChannel][pointer] < level)
    { 
      return true;
    }
  }
  
  return false;
}

bool
Simulator::fallingTrigger() const
{
  const unsigned level = m_triggerLevel & 0xf0;

  if (m_ringBuffer[m_triggerChannel][m_ringBufferPointer] <= level)
  {
    int pointer = m_ringBufferPointer;
    pointer += 1022;
    pointer %= 1024;
    
    if (m_ringBuffer[m_triggerChannel][pointer] > level)
    { 
      return true;
    }
  }
  
  return false;
}
void
Simulator::setTimeBase( Dso::TimeBase tb )
{
  //lockParam();
  
  m_timeBase = tb;
  m_step = -5;
  m_samplingFrequency = 32000000.;
  
  switch (tb)
  {
  case Dso::TB01us:
    m_timeBaseSec = 0.0000001;
    m_step = 10;
    break;
  case Dso::TB02us:
    m_timeBaseSec = 0.0000002;
    m_step = 5;
    break;
  case Dso::TB05us:
    m_timeBaseSec = 0.0000005;
    m_step = 2;
    break;
  case Dso::TB1us:
    m_timeBaseSec = 0.000001;
    m_step = 1;
    break;
  case Dso::TB2us:
    m_timeBaseSec = 0.000002;
    m_step = -2;
    break;
  case Dso::TB5us:
    m_timeBaseSec = 0.000005;
    break;
  case Dso::TB10us:
    m_samplingFrequency = 16000000.;
    m_timeBaseSec = 0.00001;
    break;
  case Dso::TB20us:
    m_samplingFrequency = 8000000.;
    m_timeBaseSec = 0.00002;
    break;
  case Dso::TB50us:
    m_samplingFrequency = 3200000.;
    m_timeBaseSec = 0.00005;
    break;
  case Dso::TB100us:
    m_samplingFrequency = 1600000.;
    m_timeBaseSec = 0.0001;
    break;
  case Dso::TB200us:
    m_samplingFrequency = 800000.;
    m_timeBaseSec = 0.0002;
    break;
  case Dso::TB500us:
    m_samplingFrequency = 320000.;
    m_timeBaseSec = 0.0005;
    break;
  case Dso::TB1ms:
    m_samplingFrequency = 160000.;
    m_timeBaseSec = 0.001;
    break;
  case Dso::TB2ms:
    m_samplingFrequency = 80000.;
    m_timeBaseSec = 0.002;
    break;
  case Dso::TB5ms:
    m_samplingFrequency = 32000.;
    m_timeBaseSec = 0.005;
    break;
  case Dso::TB10ms:
    m_samplingFrequency = 16000.;
    m_timeBaseSec = 0.01;
    break;
  case Dso::TB20ms:
    m_samplingFrequency = 8000.;
    m_timeBaseSec = 0.02;
    break;
  case Dso::TB50ms:
    m_samplingFrequency = 3200.;
    m_timeBaseSec = 0.05;
    break;
  case Dso::TB100ms:
    m_samplingFrequency = 1600.;
    m_timeBaseSec = 0.1;
    break;
  }
  
  //unlockParam();
}

void
Simulator::setVoltsDiv( int channel, Dso::VoltsDiv vd )
{
  //lockParam();
  
  m_voltsDiv[channel] = vd;
  
  switch (vd)
  {
  case Dso::VD10mv:
    m_voltsDivVolts[channel] = 0.01;
    break;
  case Dso::VD20mv:
    m_voltsDivVolts[channel] = 0.02;
    break;
  case Dso::VD50mv:
    m_voltsDivVolts[channel] = 0.05;
    break;
  case Dso::VD100mv:
    m_voltsDivVolts[channel] = 0.1;
    break;
  case Dso::VD200mv:
    m_voltsDivVolts[channel] = 0.2;
    break;
  case Dso::VD500mv:
    m_voltsDivVolts[channel] = 0.5;
    break;
  case Dso::VD1v:
    m_voltsDivVolts[channel] = 1.0;
    break;
  case Dso::VD2v:
    m_voltsDivVolts[channel] = 2.0;
    break;
  case Dso::VD5v:
    m_voltsDivVolts[channel] = 5.0;
    break;
  } 
  
  //unlockParam();
}
